/* ===========================================================================

    hello.c  - Simple "hello world" program for Axiom CML12SDP256 board

    Version:        1.0
    Author:         Dusty Nidey, Axiom Manufacturing (www.axman.com)
    Compiler:       GNU for 68HC11 & 68HC12 (www.gnu-m68hc11.org)

    This is freeware - use as you like

==============================================================================
------------------------------------------------------------------------------
*/

#include "..\ports_d256.h"

// Constant Definitions
// --------------------

#define Eclock  24000000              // 24Mhz crystal clock frequency (NOICE)
//#define Eclock  8000000               // 8Mhz crystal clock frequency (MON12)

// this board has a 4Mhz osc clock
#define CLKX    ((Eclock/1000000)/4) - 1  // pll enable for Eclock

// values for baud registers, based on clock frequency.
#define Baud115200 (Eclock/16)/115200
#define Baud57600  (Eclock/16)/57600
#define Baud38400  (Eclock/16)/38400
#define Baud19200  (Eclock/16)/19200
#define Baud9600   (Eclock/16)/9600

#define LF 10			/* line feed */
#define CR 13			/* carriage return */

// RAM Variables
// -------------
unsigned char   ioport;         // 0 = use SC0 for standard i/o.  1 = use SC1

// Function Prototypes
// -------------------
char ReadyToSend(char portnum);
char putchar(unsigned char scbyte);
void putch(unsigned char ch);
void puts(char *sptr);
void end_here();
void InitSerial(unsigned char baudrate);
void InitPLL();

//---------------------------------------------------------------------------
main(){

    // don't do this if debugging with serial debugger)
    asm("sei");  // disable interrupts
    InitPLL();  // Initialize PLL
    InitSerial(Baud9600);  // Initialize Serial Port 1
    asm("cli");  // enable interrupts

    ioport = 0;               // default output to port sci0
    puts("\nHello World!\n");
    end_here();
}

void end_here(){
    return; // this is here for a constant place to put a breakpoint
}

//---------------------------------------------------------------------------
// change the clock frequency using the PLL
void InitPLL(){
    unsigned char stat;

    // Enable the PLL
    SYNR =  CLKX;   // set Eclock frequency
    do{             // wait for PLL to lock...
        stat = CRGFLG;  // get PLL flags
    }while((stat & 0x08) == 0);

    CLKSEL = 0x80;  // turn PLL on
}

//---------------------------------------------------------------------------
// Initialize serial port
void InitSerial(unsigned char baudrate){
    SC0BDL = baudrate;  // set baud rate register
    SC0BDH = 0x00;
    SC0CR1 = 0;     // configure SCI0 control registers
    SC0CR2 = 0x0C;  // enable transmit and receive
}

/*----------------------------------------------------------------------------
ReadyToSend
-----------
 Wait for the serial port Status register to indicate the transmiter is
 ready to receive another byte.

 INPUT:     ioport      0= SCI0, 1= SCI1
 OUTPUT:    returns 0 when ready.  NO TIMEOUT is currently implemented
----------------------------------------------------------------------------*/
char ReadyToSend(char portnum){
    unsigned char Status;
    do{
        if(portnum == 0)    Status = SC0SR1;
        else                Status = SC1SR1;
    }while((Status & 0x80) == 0);
	return 0;
}

/*----------------------------------------------------------------------------
putchar
-------
 Wait for the serial port transmiter to return ready then write a single byte
 to the transmit register.

 INPUT:     SByte       Value to write
            ioport      0= SCI0, 1= SCI1
 OUTPUT:    returns 0 when finished.  NO TIMEOUT is currently implemented
----------------------------------------------------------------------------*/
char putchar(unsigned char scbyte){
//    ANDA    #$7F    ; mask parity
    if(ReadyToSend(ioport)) return 1;   // wait for ready to send
    if(ioport == 0) SC0DRL = scbyte;
    else            SC1DRL = scbyte;
	return 0;
}


/*----------------------------------------------------------------------------
putch
-----
 Send a single byte out the serial port.
 If this is the '\n' character then 2 bytes are sent which are the carriage
 return followed by line feed characters.

 INPUT:		ch			Value to write
            ioport      0= SCI0, 1= SCI1
----------------------------------------------------------------------------*/
void putch(unsigned char ch){

	if (ch == '\n'){
		putchar(CR);
		putchar(LF);
	}
	else{
		putchar(ch);
	}
}

/*----------------------------------------------------------------------------
puts
----
 Send a string of characters to the serial port.  The string must end in a
 NULL character (0x00).

 INPUT:		sptr		pointer to the string
            ioport      0= SCI0, 1= SCI1
----------------------------------------------------------------------------*/
void puts(char *sptr){
	while(*sptr){
		putch(*sptr);
		++sptr;
	}
}



