/* ===========================================================================

    timeri.c  - Simple timer interrupt program for Axiom CML12SDP256 board

    Version:        1.0
    Author:         Dusty Nidey, Axiom Manufacturing (www.axman.com)
    Compiler:       GNU for 68HC11 & 68HC12 (www.gnu-m68hc11.org)

    This is freeware - use as you like

==============================================================================
------------------------------------------------------------------------------
*/

#include "..\ports_d256.h"

// Constant Definitions
// --------------------
#define LF 10   // line feed
#define CR 13   // carriage return

// RAM Variables
// -------------
unsigned long   timeout1;       // general purpose timeout.
unsigned int    count1;         // general purpose counter
unsigned int    count4;         // general purpose counter
unsigned int    TC7Val;         // output compare timer value
unsigned char   ioport;         // 0 = use SC0 for standard i/o.  1 = use SC1
unsigned char   Sflag;

// Function Prototypes
// -------------------
char ReadyToSend(char portnum);
char putchar(unsigned char scbyte);
void putch(unsigned char ch);
void puts(char *sptr);
void end_here();
void puti(unsigned long num);
void TOC7Handle() __attribute__((interrupt));


main(){

    asm("sei");  // disable interrupts

    // Initialize Virtual Interrupt Vectors for ruhning under serial debugger (MON12/NOICE)
    *(void(**)())0x3fe0 = TOC7Handle; // set Timer Output Compare 7 vector

    // Initialize Timers

// BUS SPEED - Eclock
// On this board (CML12DP256) on power up, the bus clock defaults to 2Mhz (osc / 2)
//                            operating MON12 debugger the bus clock is running at 8Mhz
//                            operating NoICE kernel the bus clock is running at 24Mhz

// TMSK2 (now called TSCR2 in the new datasheet) changes the timer PRESCALER
//    TMSK2 = 0x00;   // Prescaler divide by 1 (TSCR2)   - makes timer system run at Bus speed exactly (no prescaler)
    TMSK2 = 0x01;   // Prescaler divide by 2 (TSCR2)   - makes timer system run at Bus speed / 2
//    TMSK2 = 0x03;   // Prescaler divide by 8 (TSCR2)  - makes timer system run at Bus speed / 8
//    TMSK2 = 0x04;   // Prescaler divide by 16 (TSCR2) - makes timer system run at Bus speed / 16 (ie. 24/16 = 1.5)
//    TMSK2 = 0x05;   // Prescaler divide by 32 (TSCR2)   - makes timer system run at Bus speed / 32
    TSCR = 0x80;    // enable timer
/*
 1/bus speed = time period (half cycle time)

 IF USING NO PRESCALER:
    1/24 = 0.041667 (41.7 nano-seconds)
    we want 500us, so 500 / 0.041667 = 12000
    this will give us a 500us time period (half cycle time)

 IF USING divide by 2 prescaler (TMSK2=0x01)
    1/12 = 0.083333 (83.3 nano-seconds)
    we want 500us, so 500 / 0.083333 = 6000
    this will give us a 500us time period (half cycle time)
*/
//    TC7Val = 12000;          // output compare time with NO prescaler (24mhz)
    TC7Val = 6000;          // output compare time with /2 prescaler
    TC7 = TCNT + TC7Val;   // Set Timer 7 Output compare Value

    TIOS = 0x80;    // set channel 7 to output compare
    TCTL1 = 0x40;   // toggle OC7 output line on output compare
    TMSK1 = 0x80;   // enable Channel 7 Interrupt Request (TIE)
    TFLG1 = 0x80;   // clear C7 output capture flag - IDLE
    TFLG2 = 0x00;   // clear timer overflow flag

    // reset all counters and flags
    count1=0;
	count4=0;
    timeout1=0;
    Sflag=0;

    asm("cli");  // enable interrupts

    ioport = 0;               // default output to port sci0

    puts("\nYou should see PT7 toggling at 500us ");

    // endless loop
    for( ; ;){
        if(Sflag == 1){ // if time expired
            Sflag=0;
            puti(count1);   // output the current count to serial port
            putch('\n');
            ++count1;
        }
    }

    end_here();
}

void end_here(){
    return; // this is here for a constant place to put a breakpoint
}

/*----------------------------------------------------------------------------
ReadyToSend
-----------
 Wait for the serial port Status register to indicate the transmiter is
 ready to receive another byte.

 INPUT:     ioport      0= SCI0, 1= SCI1
 OUTPUT:    returns 0 when ready.  NO TIMEOUT is currently implemented
----------------------------------------------------------------------------*/
char ReadyToSend(char portnum){
    unsigned char Status;
    do{
        if(portnum == 0)    Status = SC0SR1;
        else                Status = SC1SR1;
    }while((Status & 0x80) == 0);
	return 0;
}

/*----------------------------------------------------------------------------
putchar
-------
 Wait for the serial port transmiter to return ready then write a single byte
 to the transmit register.

 INPUT:     SByte       Value to write
            ioport      0= SCI0, 1= SCI1
 OUTPUT:    returns 0 when finished.  NO TIMEOUT is currently implemented
----------------------------------------------------------------------------*/
char putchar(unsigned char scbyte){
//    ANDA    #$7F    ; mask parity
    if(ReadyToSend(ioport)) return 1;   // wait for ready to send
    if(ioport == 0) SC0DRL = scbyte;
    else            SC1DRL = scbyte;
	return 0;
}


/*----------------------------------------------------------------------------
putch
-----
 Send a single byte out the serial port.
 If this is the '\n' character then 2 bytes are sent which are the carriage
 return followed by line feed characters.

 INPUT:		ch			Value to write
            ioport      0= SCI0, 1= SCI1
----------------------------------------------------------------------------*/
void putch(unsigned char ch){

	if (ch == '\n'){
		putchar(CR);
        putchar(LF);
	}
	else{
		putchar(ch);
	}
}

/*----------------------------------------------------------------------------
puts
----
 Send a string of characters to the serial port.  The string must end in a
 NULL character (0x00).

 INPUT:		sptr		pointer to the string
            ioport      0= SCI0, 1= SCI1
----------------------------------------------------------------------------*/
void puts(char *sptr){
	while(*sptr){
		putch(*sptr);
		++sptr;
	}
}

/*----------------------------------------------------------------------------
strlength
---------
 Returns the number of bytes in a 0 terminated string.
 The 0 is not counted.

 INPUT:		sptr		pointer to the string
 OUTPUT:	returns the length of the string
----------------------------------------------------------------------------*/
unsigned short strlength(char *sptr){
    unsigned short strlencount;
	strlencount=0;
	while(*sptr != 0){
		++strlencount;
		++sptr;
	}
	return(strlencount);
}
/*----------------------------------------------------------------------------
reverse
-------
 Reverse the character order of a 0 terminating string

 INPUT:		rptr		pointer to the string
 OUTPUT:	rptr		new string
----------------------------------------------------------------------------*/
void reverse(char *rptr){
    unsigned short countb, counte, middle;
	char tbyte;

	counte = (strlength(rptr)) - 1;  // get number of digits in the string
	for(countb=0; countb < counte; countb++, counte--){
		tbyte = *(rptr+countb);
		*(rptr+countb) = *(rptr+counte);
		*(rptr+counte) = tbyte;
	}
}

/*----------------------------------------------------------------------------
itoa
----
 Convert a number to a 0 terminating ASCII string representing a Base 10
 Decimal Value

 INPUT:		n		number to convert
 			s		pointer to the array to hold the string
----------------------------------------------------------------------------*/
void itoa(unsigned long n, char *s){
  unsigned long sign;
  char *ptr;

  ptr = s;
//  sign = n;
//  if (sign < 0) n = -n;
  do {
    *ptr++ = n % 10 + '0';
    } while ((n = n / 10) > 0);
//  if (sign < 0) *ptr++ = '-';
  *ptr = '\0';
  reverse(s);
}

/*----------------------------------------------------------------------------
puti
----
 Output an ASCII Decimal string representation of a number to the serial port

 INPUT:		num		number to send
----------------------------------------------------------------------------*/
void puti(unsigned long num){
	char tempbuf[7];

	itoa(num, tempbuf);
	puts(tempbuf);
}


//=== INTERRUPT ROUTINES =====================================================

/*----------------------------------------------------------------------------
TOC7Handle
----------
 Interrupt Handler routine for TOC7
----------------------------------------------------------------------------*/
void __attribute__((interrupt))
TOC7Handle(){
    // here every 500us
    TFLG1 = 0x80;   // clear C7 output capture flag - IDLE
    TC7 = TC7 + TC7Val;   // Set new Timer 7 Output compare Value

    ++timeout1; // incriment timeout counter
    if(++count4 > 2000){  // if time to update counters
        // here every 1 second
		count4=0;
        Sflag=1;
	}
}

