/* ===========================================================================

    keypad.c  - KEYPAD routines for Axiom CML12SDP256 and 912x boards using
                port H

    Version:        1.0
    Author:         Dusty Nidey, Axiom Manufacturing (www.axman.com)
    Compiler:       GNU for 68HC11 & 68HC12 (www.gnu-m68hc11.org)

    This is freeware - use as you like

==============================================================================
------------------------------------------------------------------------------
*/

#include "..\ports_d256.h"
#include "keypad.h"

// Constant Definitions
// --------------------
const char *keytable = ".123A456B789C*0#D";

// RAM Variables
// -------------
unsigned char Newkey, Oldkey;


/*----------------------------------------------------------------------------
InitKeyPad
----------
 Initialize KEYPAD port used

     Port H 0-3 must be output
            4-7 must be input with pull-down enabled
 ----------------------------------------------------------------------------*/
void InitKeypad(){
    DDRH = 0x0F;    // set Port H direction (0=input, 1=output)
    PPSH = 0xF0;    // set for pull downs
    PERH = 0xF0;    // enable pull downs
    Newkey=0; Oldkey=0; // reset keypad status variables
}


/*----------------------------------------------------------------------------
kbhit
-----
 Does a KEYPAD scan.

 returns 0 if NO key is pressed
 otherwise returns the representation of the key pressed from the included table

 ----------------------------------------------------------------------------*/
char kbhit(){
    unsigned char column, keycount, kstat, delaytime;

    PTH = 0x0F;     // set all KEYPAD columns
    kstat = PTH;    // get KEYPAD data
    kstat &= 0xF0;  // mask rows
    if(kstat != 0){ // if key pressed

        for(column = 0x01, keycount=0; column < 0x10; column <<= 1, keycount++){
            PTH = 0;    // clear all KEYPAD columns
            PTH = column;   // set this KEYPAD column
            for(delaytime=0x20; delaytime > 0; delaytime--) ;   // wait for lines to settle
            kstat = PTH;    // get KEYPAD data
            kstat &= 0xF0;  // mask rows
            if(kstat != 0){ // if key pressed on this column
                kstat >>= 4;
                if(kstat == 0x04)   kstat = 3;
                if(kstat == 0x08)   kstat = 4;
                return(keytable[(keycount*4) + kstat]);
            }
        }
    }
    return 0;   // return no key pressed

}

/*----------------------------------------------------------------------------
getkeypad
---------
 Returns a key pressed on the keypad.  First waits for any current key pressed
 to be released, then does not return until a new key is debounced.

 Returns the representation of the key pressed from the array keytable[]
 ----------------------------------------------------------------------------*/
char getkeypad(){
    char debcount;  // debounce counter

    do{
        Newkey = kbhit();   // get key pressed status
    }while(Newkey == Oldkey);   // while same key pressed as last scan

    for(debcount=0; debcount < KEY_DEBTIME; debcount++){
        Newkey = kbhit();
        if(Newkey == 0 || Newkey != Oldkey){    // if key not still pressed
            debcount=0; // clear debounce counter
        }
        Oldkey = Newkey;    // save for next scan
    }
    return(Newkey);
}

